package quimufu.simple_creator;

import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonObject;
import net.fabricmc.fabric.api.itemgroup.v1.ItemGroupEvents;
import net.fabricmc.fabric.api.object.builder.v1.block.FabricBlockSettings;
import net.fabricmc.fabric.api.registry.FlammableBlockRegistry;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.MapColor;
import net.minecraft.block.enums.NoteBlockInstrument;
import net.minecraft.block.piston.PistonBehavior;
import net.minecraft.entity.EntityType;
import net.minecraft.item.BlockItem;
import net.minecraft.item.Item;
import net.minecraft.item.ItemGroup;
import net.minecraft.loot.LootTable;
import net.minecraft.registry.Registries;
import net.minecraft.registry.Registry;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.sound.BlockSoundGroup;
import net.minecraft.util.Identifier;
import net.minecraft.util.Pair;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.BlockView;
import org.apache.logging.log4j.Level;

import java.util.Map;

import static quimufu.simple_creator.SimpleCreatorMod.log;


public class BlockResourceLoader extends GenericManualResourceLoader<Pair<Block, Item>> {
    private static final Gson GSON = (new GsonBuilder()).setPrettyPrinting().disableHtmlEscaping().create();
    private static final String dataType = "blocks";
    public static Map<Identifier, Block> blocks = Maps.newHashMap();
    public static Map<Identifier, Item> blockItems = Maps.newHashMap();

    BlockResourceLoader() {
        super(GSON, dataType);
    }

    @Override
    protected void register(Identifier id, Pair<Block, Item> thing) {
        Registry.register(Registries.BLOCK, id, thing.getLeft());
        Registry.register(Registries.ITEM, id, thing.getRight());
    }

    @Override
    protected void save(Identifier id, Pair<Block, Item> thing) {
        blocks.put(id, thing.getLeft());
        blockItems.put(id, thing.getRight());
    }

    @Override
    protected Pair<Block, Item> deserialize(Pair<Identifier, JsonObject> e) {
        JsonObject jo = e.getRight();
        log(Level.INFO, e.getRight().toString());

        // get block information
        BlockSettingsPojo bspj = GSON.fromJson(jo, BlockSettingsPojo.class);

        // move block information in Block.Settings (!!hacky!!)
        Block.Settings bs = getSettings(bspj);

        // parse item group
        String group = bspj.itemGroup;
        RegistryKey<ItemGroup> g = ItemResourceLoader.findGroup(group);
        //create block and corresponding item
        Block resB = new Block(bs);
        Item resI = new BlockItem(resB, new Item.Settings());
        ItemGroupEvents.modifyEntriesEvent(g).register(content -> content.add(resI));

        int burnChance = bspj.burnChance;
        int spreadChance = bspj.spreadChance;
        if (burnChance != -1 && spreadChance != -1) {
            FlammableBlockRegistry.getDefaultInstance().add(resB, spreadChance, burnChance);
        }

        SimpleCreatorMod.BLOCKS_RENDER_LAYER.add(new Pair<>(resB, bspj.renderLayer));
        return new Pair<>(resB, resI);
    }

    private Block.Settings getSettings(BlockSettingsPojo bspj) {
        FabricBlockSettings fabricBlockSettings = FabricBlockSettings.create();

        fabricBlockSettings
                .collidable(bspj.collidable)
                .slipperiness(bspj.slipperiness)
                .velocityMultiplier(bspj.movementVelocityMultiplier)
                .jumpVelocityMultiplier(bspj.jumpVelocityMultiplier)
                .sounds(getSoundGroup(bspj.soundGroup))
                .drops(getDropTableId(bspj.dropTableId))
                .mapColor(getMapColor(bspj.mapColor))
                .allowsSpawning(bspj.allowsSpawning ? BlockResourceLoader::always : BlockResourceLoader::never)
                .solidBlock(bspj.solidBlock ? BlockResourceLoader::always : BlockResourceLoader::never)
                .suffocates(bspj.suffocates ? BlockResourceLoader::always : BlockResourceLoader::never)
                .blockVision(bspj.blockVision ? BlockResourceLoader::always : BlockResourceLoader::never)
                .postProcess(bspj.postProcess ? BlockResourceLoader::always : BlockResourceLoader::never)
                .emissiveLighting(bspj.emissiveLighting ? BlockResourceLoader::always : BlockResourceLoader::never)
                .hardness(bspj.hardness)
                .resistance(bspj.explosionResistance)
                .offset(getOffset(bspj.modelOffset))
                .pistonBehavior(getPistonBehavior(bspj.pistonBehavior))
                .instrument(getInstrument(bspj.instrument))
                .luminance(bspj.lightLevel);
        if (bspj.burnable) {
            fabricBlockSettings.burnable();
        }
        //not supported for now
        //fabricBlockSettings.liquid()
        if (bspj.solidBlock) {
            fabricBlockSettings.solid();
        } else {
            fabricBlockSettings.notSolid();
        }
        if (bspj.replaceable) {
            fabricBlockSettings.replaceable();
        }
        //not supported for now
        //fabricBlockSettings.air()

        if (bspj.noBlockBreakParticles) {
            fabricBlockSettings.noBlockBreakParticles();
        }
        if (bspj.requiresTool) {
            fabricBlockSettings.requiresTool();

        }
        if (bspj.breaksInstantly) {
            fabricBlockSettings.breakInstantly();
        }

        if (!bspj.opaque) {
            fabricBlockSettings.nonOpaque();
        }

        return fabricBlockSettings;
    }

    private NoteBlockInstrument getInstrument(String instrument) {
        return switch (instrument.toUpperCase()) {
            case "HARP" -> NoteBlockInstrument.HARP;
            case "BASEDRUM" -> NoteBlockInstrument.BASEDRUM;
            case "SNARE" -> NoteBlockInstrument.SNARE;
            case "HAT" -> NoteBlockInstrument.HAT;
            case "BASS" -> NoteBlockInstrument.BASS;
            case "FLUTE" -> NoteBlockInstrument.FLUTE;
            case "BELL" -> NoteBlockInstrument.BELL;
            case "GUITAR" -> NoteBlockInstrument.GUITAR;
            case "CHIME" -> NoteBlockInstrument.CHIME;
            case "XYLOPHONE" -> NoteBlockInstrument.XYLOPHONE;
            case "IRON_XYLOPHONE" -> NoteBlockInstrument.IRON_XYLOPHONE;
            case "COW_BELL" -> NoteBlockInstrument.COW_BELL;
            case "DIDGERIDOO" -> NoteBlockInstrument.DIDGERIDOO;
            case "BIT" -> NoteBlockInstrument.BIT;
            case "BANJO" -> NoteBlockInstrument.BANJO;
            case "PLING" -> NoteBlockInstrument.PLING;
            case "ZOMBIE" -> NoteBlockInstrument.ZOMBIE;
            case "SKELETON" -> NoteBlockInstrument.SKELETON;
            case "CREEPER" -> NoteBlockInstrument.CREEPER;
            case "DRAGON" -> NoteBlockInstrument.DRAGON;
            case "WITHER_SKELETON" -> NoteBlockInstrument.WITHER_SKELETON;
            case "PIGLIN" -> NoteBlockInstrument.PIGLIN;
            case "CUSTOM_HEAD" -> NoteBlockInstrument.CUSTOM_HEAD;
            default -> {
                log(Level.WARN, "Instrument " + instrument + " not found, using harp");
                yield NoteBlockInstrument.HARP;
            }
        };

    }

    private AbstractBlock.OffsetType getOffset(String modelOffset) {
        return switch (modelOffset.toUpperCase()) {
            case "NONE" -> AbstractBlock.OffsetType.NONE;
            case "XZ" -> AbstractBlock.OffsetType.XZ;
            case "XYZ" -> AbstractBlock.OffsetType.XYZ;
            default -> {
                log(Level.WARN, "ModelOffset " + modelOffset + " not found, using none");
                yield AbstractBlock.OffsetType.NONE;
            }
        };

    }

    private static boolean always(BlockState blockState, BlockView blockView, BlockPos blockPos) {
        return true;
    }

    private static boolean never(BlockState blockState, BlockView blockView, BlockPos blockPos) {
        return false;
    }

    /**
     * A shortcut to always return {@code false} in a typed context predicate with an
     * {@link EntityType}, used like {@code settings.allowSpawning(Blocks::never)}.
     */
    private static Boolean never(BlockState state, BlockView world, BlockPos pos, EntityType<?> type) {
        return false;
    }

    /**
     * A shortcut to always return {@code true} in a typed context predicate with an
     * {@link EntityType}, used like {@code settings.allowSpawning(Blocks::always)}.
     */
    private static Boolean always(BlockState state, BlockView world, BlockPos pos, EntityType<?> type) {
        return true;
    }

    private PistonBehavior getPistonBehavior(String pistonBehavior) {
        return switch (pistonBehavior.toUpperCase()) {
            case "NORMAL" -> PistonBehavior.NORMAL;
            case "DESTROY" -> PistonBehavior.DESTROY;
            case "BLOCK" -> PistonBehavior.BLOCK;
            case "IGNORE" -> PistonBehavior.IGNORE;
            case "PUSH_ONLY" -> PistonBehavior.PUSH_ONLY;
            default -> {
                log(Level.WARN, "Piston Behavior " + pistonBehavior + " not found, using normal");
                yield PistonBehavior.NORMAL;
            }
        };
    }


    private MapColor getMapColor(String color) {
        return switch (color.toUpperCase()) {
            case "CLEAR" -> MapColor.CLEAR;
            case "PALE_GREEN" -> MapColor.PALE_GREEN;
            case "PALE_YELLOW" -> MapColor.PALE_YELLOW;
            case "WHITE_GRAY" -> MapColor.WHITE_GRAY;
            case "BRIGHT_RED" -> MapColor.BRIGHT_RED;
            case "PALE_PURPLE" -> MapColor.PALE_PURPLE;
            case "IRON_GRAY" -> MapColor.IRON_GRAY;
            case "DARK_GREEN" -> MapColor.DARK_GREEN;
            case "WHITE" -> MapColor.WHITE;
            case "LIGHT_BLUE_GRAY" -> MapColor.LIGHT_BLUE_GRAY;
            case "DIRT_BROWN" -> MapColor.DIRT_BROWN;
            case "STONE_GRAY" -> MapColor.STONE_GRAY;
            case "WATER_BLUE" -> MapColor.WATER_BLUE;
            case "OAK_TAN" -> MapColor.OAK_TAN;
            case "OFF_WHITE" -> MapColor.OFF_WHITE;
            case "ORANGE" -> MapColor.ORANGE;
            case "MAGENTA" -> MapColor.MAGENTA;
            case "LIGHT_BLUE" -> MapColor.LIGHT_BLUE;
            case "YELLOW" -> MapColor.YELLOW;
            case "LIME" -> MapColor.LIME;
            case "PINK" -> MapColor.PINK;
            case "GRAY" -> MapColor.GRAY;
            case "LIGHT_GRAY" -> MapColor.LIGHT_GRAY;
            case "CYAN" -> MapColor.CYAN;
            case "PURPLE" -> MapColor.PURPLE;
            case "BLUE" -> MapColor.BLUE;
            case "BROWN" -> MapColor.BROWN;
            case "GREEN" -> MapColor.GREEN;
            case "RED" -> MapColor.RED;
            case "BLACK" -> MapColor.BLACK;
            case "GOLD" -> MapColor.GOLD;
            case "DIAMOND_BLUE" -> MapColor.DIAMOND_BLUE;
            case "LAPIS_BLUE" -> MapColor.LAPIS_BLUE;
            case "EMERALD_GREEN" -> MapColor.EMERALD_GREEN;
            case "SPRUCE_BROWN" -> MapColor.SPRUCE_BROWN;
            case "DARK_RED" -> MapColor.DARK_RED;
            case "TERRACOTTA_WHITE" -> MapColor.TERRACOTTA_WHITE;
            case "TERRACOTTA_ORANGE" -> MapColor.TERRACOTTA_ORANGE;
            case "TERRACOTTA_MAGENTA" -> MapColor.TERRACOTTA_MAGENTA;
            case "TERRACOTTA_LIGHT_BLUE" -> MapColor.TERRACOTTA_LIGHT_BLUE;
            case "TERRACOTTA_YELLOW" -> MapColor.TERRACOTTA_YELLOW;
            case "TERRACOTTA_LIME" -> MapColor.TERRACOTTA_LIME;
            case "TERRACOTTA_PINK" -> MapColor.TERRACOTTA_PINK;
            case "TERRACOTTA_GRAY" -> MapColor.TERRACOTTA_GRAY;
            case "TERRACOTTA_LIGHT_GRAY" -> MapColor.TERRACOTTA_LIGHT_GRAY;
            case "TERRACOTTA_CYAN" -> MapColor.TERRACOTTA_CYAN;
            case "TERRACOTTA_PURPLE" -> MapColor.TERRACOTTA_PURPLE;
            case "TERRACOTTA_BLUE" -> MapColor.TERRACOTTA_BLUE;
            case "TERRACOTTA_BROWN" -> MapColor.TERRACOTTA_BROWN;
            case "TERRACOTTA_GREEN" -> MapColor.TERRACOTTA_GREEN;
            case "TERRACOTTA_RED" -> MapColor.TERRACOTTA_RED;
            case "TERRACOTTA_BLACK" -> MapColor.TERRACOTTA_BLACK;
            case "DULL_RED" -> MapColor.DULL_RED;
            case "DULL_PINK" -> MapColor.DULL_PINK;
            case "DARK_CRIMSON" -> MapColor.DARK_CRIMSON;
            case "TEAL" -> MapColor.TEAL;
            case "DARK_AQUA" -> MapColor.DARK_AQUA;
            case "DARK_DULL_PINK" -> MapColor.DARK_DULL_PINK;
            case "BRIGHT_TEAL" -> MapColor.BRIGHT_TEAL;
            case "DEEPSLATE_GRAY" -> MapColor.DEEPSLATE_GRAY;
            case "RAW_IRON_PINK" -> MapColor.RAW_IRON_PINK;
            case "LICHEN_GREEN" -> MapColor.LICHEN_GREEN;
            default -> {
                log(Level.WARN, "MapColor " + color + " not found, using pink");
                yield MapColor.PINK;
            }
        };
    }

    private RegistryKey<LootTable> getDropTableId(String s) {
        if (s == null)
            return null;
        Identifier i = Identifier.tryParse(s);
        if (i == null) {
            log(Level.WARN, "Drop table invalid " + s + ", using default");
        }
        return RegistryKey.of(RegistryKeys.LOOT_TABLE, i);
    }

    private BlockSoundGroup getSoundGroup(String s) {
        return switch (s.toUpperCase()) {
            case "INTENTIONALLY_EMPTY" -> BlockSoundGroup.INTENTIONALLY_EMPTY;
            case "WOOD" -> BlockSoundGroup.WOOD;
            case "GRAVEL" -> BlockSoundGroup.GRAVEL;
            case "GRASS" -> BlockSoundGroup.GRASS;
            case "LILY_PAD" -> BlockSoundGroup.LILY_PAD;
            case "STONE" -> BlockSoundGroup.STONE;
            case "METAL" -> BlockSoundGroup.METAL;
            case "GLASS" -> BlockSoundGroup.GLASS;
            case "WOOL" -> BlockSoundGroup.WOOL;
            case "SAND" -> BlockSoundGroup.SAND;
            case "SNOW" -> BlockSoundGroup.SNOW;
            case "POWDER_SNOW" -> BlockSoundGroup.POWDER_SNOW;
            case "LADDER" -> BlockSoundGroup.LADDER;
            case "ANVIL" -> BlockSoundGroup.ANVIL;
            case "SLIME" -> BlockSoundGroup.SLIME;
            case "HONEY" -> BlockSoundGroup.HONEY;
            case "WET_GRASS" -> BlockSoundGroup.WET_GRASS;
            case "CORAL" -> BlockSoundGroup.CORAL;
            case "BAMBOO" -> BlockSoundGroup.BAMBOO;
            case "BAMBOO_SAPLING" -> BlockSoundGroup.BAMBOO_SAPLING;
            case "SCAFFOLDING" -> BlockSoundGroup.SCAFFOLDING;
            case "SWEET_BERRY_BUSH" -> BlockSoundGroup.SWEET_BERRY_BUSH;
            case "CROP" -> BlockSoundGroup.CROP;
            case "STEM" -> BlockSoundGroup.STEM;
            case "VINE" -> BlockSoundGroup.VINE;
            case "NETHER_WART" -> BlockSoundGroup.NETHER_WART;
            case "LANTERN" -> BlockSoundGroup.LANTERN;
            case "NETHER_STEM" -> BlockSoundGroup.NETHER_STEM;
            case "NYLIUM" -> BlockSoundGroup.NYLIUM;
            case "FUNGUS" -> BlockSoundGroup.FUNGUS;
            case "ROOTS" -> BlockSoundGroup.ROOTS;
            case "SHROOMLIGHT" -> BlockSoundGroup.SHROOMLIGHT;
            case "WEEPING_VINES" -> BlockSoundGroup.WEEPING_VINES;
            case "WEEPING_VINES_LOW_PITCH" -> BlockSoundGroup.WEEPING_VINES_LOW_PITCH;
            case "SOUL_SAND" -> BlockSoundGroup.SOUL_SAND;
            case "SOUL_SOIL" -> BlockSoundGroup.SOUL_SOIL;
            case "BASALT" -> BlockSoundGroup.BASALT;
            case "WART_BLOCK" -> BlockSoundGroup.WART_BLOCK;
            case "NETHERRACK" -> BlockSoundGroup.NETHERRACK;
            case "NETHER_BRICKS" -> BlockSoundGroup.NETHER_BRICKS;
            case "NETHER_SPROUTS" -> BlockSoundGroup.NETHER_SPROUTS;
            case "NETHER_ORE" -> BlockSoundGroup.NETHER_ORE;
            case "BONE" -> BlockSoundGroup.BONE;
            case "NETHERITE" -> BlockSoundGroup.NETHERITE;
            case "ANCIENT_DEBRIS" -> BlockSoundGroup.ANCIENT_DEBRIS;
            case "LODESTONE" -> BlockSoundGroup.LODESTONE;
            case "CHAIN" -> BlockSoundGroup.CHAIN;
            case "NETHER_GOLD_ORE" -> BlockSoundGroup.NETHER_GOLD_ORE;
            case "GILDED_BLACKSTONE" -> BlockSoundGroup.GILDED_BLACKSTONE;
            case "CANDLE" -> BlockSoundGroup.CANDLE;
            case "AMETHYST_BLOCK" -> BlockSoundGroup.AMETHYST_BLOCK;
            case "AMETHYST_CLUSTER" -> BlockSoundGroup.AMETHYST_CLUSTER;
            case "SMALL_AMETHYST_BUD" -> BlockSoundGroup.SMALL_AMETHYST_BUD;
            case "MEDIUM_AMETHYST_BUD" -> BlockSoundGroup.MEDIUM_AMETHYST_BUD;
            case "LARGE_AMETHYST_BUD" -> BlockSoundGroup.LARGE_AMETHYST_BUD;
            case "TUFF" -> BlockSoundGroup.TUFF;
            case "TUFF_BRICKS" -> BlockSoundGroup.TUFF_BRICKS;
            case "POLISHED_TUFF" -> BlockSoundGroup.POLISHED_TUFF;
            case "CALCITE" -> BlockSoundGroup.CALCITE;
            case "DRIPSTONE_BLOCK" -> BlockSoundGroup.DRIPSTONE_BLOCK;
            case "POINTED_DRIPSTONE" -> BlockSoundGroup.POINTED_DRIPSTONE;
            case "COPPER" -> BlockSoundGroup.COPPER;
            case "COPPER_BULB" -> BlockSoundGroup.COPPER_BULB;
            case "COPPER_GRATE" -> BlockSoundGroup.COPPER_GRATE;
            case "CAVE_VINES" -> BlockSoundGroup.CAVE_VINES;
            case "SPORE_BLOSSOM" -> BlockSoundGroup.SPORE_BLOSSOM;
            case "AZALEA" -> BlockSoundGroup.AZALEA;
            case "FLOWERING_AZALEA" -> BlockSoundGroup.FLOWERING_AZALEA;
            case "MOSS_CARPET" -> BlockSoundGroup.MOSS_CARPET;
            case "PINK_PETALS" -> BlockSoundGroup.PINK_PETALS;
            case "MOSS_BLOCK" -> BlockSoundGroup.MOSS_BLOCK;
            case "BIG_DRIPLEAF" -> BlockSoundGroup.BIG_DRIPLEAF;
            case "SMALL_DRIPLEAF" -> BlockSoundGroup.SMALL_DRIPLEAF;
            case "ROOTED_DIRT" -> BlockSoundGroup.ROOTED_DIRT;
            case "HANGING_ROOTS" -> BlockSoundGroup.HANGING_ROOTS;
            case "AZALEA_LEAVES" -> BlockSoundGroup.AZALEA_LEAVES;
            case "SCULK_SENSOR" -> BlockSoundGroup.SCULK_SENSOR;
            case "SCULK_CATALYST" -> BlockSoundGroup.SCULK_CATALYST;
            case "SCULK" -> BlockSoundGroup.SCULK;
            case "SCULK_VEIN" -> BlockSoundGroup.SCULK_VEIN;
            case "SCULK_SHRIEKER" -> BlockSoundGroup.SCULK_SHRIEKER;
            case "GLOW_LICHEN" -> BlockSoundGroup.GLOW_LICHEN;
            case "DEEPSLATE" -> BlockSoundGroup.DEEPSLATE;
            case "DEEPSLATE_BRICKS" -> BlockSoundGroup.DEEPSLATE_BRICKS;
            case "DEEPSLATE_TILES" -> BlockSoundGroup.DEEPSLATE_TILES;
            case "POLISHED_DEEPSLATE" -> BlockSoundGroup.POLISHED_DEEPSLATE;
            case "FROGLIGHT" -> BlockSoundGroup.FROGLIGHT;
            case "FROGSPAWN" -> BlockSoundGroup.FROGSPAWN;
            case "MANGROVE_ROOTS" -> BlockSoundGroup.MANGROVE_ROOTS;
            case "MUDDY_MANGROVE_ROOTS" -> BlockSoundGroup.MUDDY_MANGROVE_ROOTS;
            case "MUD" -> BlockSoundGroup.MUD;
            case "MUD_BRICKS" -> BlockSoundGroup.MUD_BRICKS;
            case "PACKED_MUD" -> BlockSoundGroup.PACKED_MUD;
            case "HANGING_SIGN" -> BlockSoundGroup.HANGING_SIGN;
            case "NETHER_WOOD_HANGING_SIGN" -> BlockSoundGroup.NETHER_WOOD_HANGING_SIGN;
            case "BAMBOO_WOOD_HANGING_SIGN" -> BlockSoundGroup.BAMBOO_WOOD_HANGING_SIGN;
            case "BAMBOO_WOOD" -> BlockSoundGroup.BAMBOO_WOOD;
            case "NETHER_WOOD" -> BlockSoundGroup.NETHER_WOOD;
            case "CHERRY_WOOD" -> BlockSoundGroup.CHERRY_WOOD;
            case "CHERRY_SAPLING" -> BlockSoundGroup.CHERRY_SAPLING;
            case "CHERRY_LEAVES" -> BlockSoundGroup.CHERRY_LEAVES;
            case "CHERRY_WOOD_HANGING_SIGN" -> BlockSoundGroup.CHERRY_WOOD_HANGING_SIGN;
            case "CHISELED_BOOKSHELF" -> BlockSoundGroup.CHISELED_BOOKSHELF;
            case "SUSPICIOUS_SAND" -> BlockSoundGroup.SUSPICIOUS_SAND;
            case "SUSPICIOUS_GRAVEL" -> BlockSoundGroup.SUSPICIOUS_GRAVEL;
            case "DECORATED_POT" -> BlockSoundGroup.DECORATED_POT;
            case "DECORATED_POT_SHATTER" -> BlockSoundGroup.DECORATED_POT_SHATTER;
            case "TRIAL_SPAWNER" -> BlockSoundGroup.TRIAL_SPAWNER;
            case "SPONGE" -> BlockSoundGroup.SPONGE;
            case "WET_SPONGE" -> BlockSoundGroup.WET_SPONGE;
            case "VAULT" -> BlockSoundGroup.VAULT;
            case "HEAVY_CORE" -> BlockSoundGroup.HEAVY_CORE;
            case "COBWEB" -> BlockSoundGroup.COBWEB;
            default -> {
                log(Level.WARN, "Sound group " + s + " not found, using stone");
                yield BlockSoundGroup.STONE;
            }
        };
    }


}
